rm(list=ls(all=TRUE))
options(echo  =  TRUE)

library(iRRA)

rocc <- function(dataframe, ind1, ind2) { 
  dataframe <- dataframe[order(dataframe[ind1], decreasing = TRUE),]  
  predictions <- dataframe[[ind1]] 
  labels <- dataframe[[ind2]] 
  x<- c(0) 
  y<-c(0) 
  for(i in 1:length(predictions)){ 
    TP <- 0 
    TN <- 0 
    FP <- 0 
    FN <- 0 
    if(i!=1){ 
      if(predictions[i] == predictions[i-1]){ 
        next 
      } 
    } 
    for(j in 1:length(predictions)){ 
      if(predictions[j] < predictions[i]){ 
        if(labels[j] == 0){ 
          TN <- (TN+1) 
        }else{ 
          FN <- (FN+1) 
        } 
      }else{ 
        if(labels[j] == 0){ 
          FP <- (FP+1) 
        }else{ 
          TP <- (TP+1) 
        } 
      } 
    } 
    TPR <- TP/(TP+FN) 
    FPR <- 1 - TN/(TN+FP) 
    x <- c(x,FPR) 
    y <- c(y,TPR) 
  } 
  x<-c(x,1) 
  y<-c(y,1) 
  roc <- list(x,y) 
  names(roc) <- c("x", "y") 
  invisible(roc) 
}

data1=read.csv("data_for_ROC1.csv")
n=nrow((data1))
AP=sum(data1$labels)
AN=n-AP
firstROCcurve <- rocc(data1, 1, 2)
rra_result1 <- rra(firstROCcurve$x, firstROCcurve$y, AP, AN, plot = FALSE)
rra.plot(rra_result1, cROI = "white", cUnder = "white", main="")

data2=read.csv("data_for_ROC2.csv")
n=nrow((data2))
AP=sum(data2$labels)
AN=n-AP
secondROCcurve <- rocc(data2, 1, 2)
rra_result2 <- rra(secondROCcurve$x, secondROCcurve$y, AP, AN, plot = FALSE)
rra.plot(rra_result2, cROI = "white", cUnder = "white", main="")

dataBerek=read.csv("berek.csv")
dataBerek$bug=ifelse(dataBerek$bug>0,1,0)
n=nrow((dataBerek))
AP=sum(dataBerek$bug)
AN=n-AP
berekWMCROCcurve <- rocc(dataBerek, 4, 24)
rocWMC <- rra(berekWMCROCcurve$x, berekWMCROCcurve$y, AP, AN, plot = FALSE)
rra.plot(rocWMC, cROI = "white", cUnder = "white", main="")

berekLOCROCcurve <- rocc(dataBerek, 14, 24)
rocLOC <- rra(berekLOCROCcurve$x, berekLOCROCcurve$y, AP, AN, plot = FALSE)
rra.plot(rocLOC, cROI = "white", cUnder = "white", main="")

berekACROCcurve <- rocc(dataBerek, 10, 24)
rocAC <- rra(berekACROCcurve$x, berekACROCcurve$y, AP, AN, plot = FALSE)
rra.plot(rocAC, cROI = "white", cUnder = "white", main="")

n=nrow((data1))
AP=sum(data1$labels)
AN=n-AP
temp=rra(firstROCcurve$x, firstROCcurve$y, AP, AN, recall=TRUE, fallout = TRUE)

rra(firstROCcurve$x, firstROCcurve$y, AP, AN, phi=TRUE, c_phi=0.5)

n=nrow((data2))
AP=sum(data2$labels)
AN=n-AP
rra(secondROCcurve$x, secondROCcurve$y, AP, AN, j=TRUE, c_j=0.3, 
    markedness=TRUE, c_markedness=0.4, cUnder="green", main = "J, markedness and phi constrain") 

rra(secondROCcurve$x, secondROCcurve$y, AP, AN, ncost=TRUE, lambda=0.7, mu=0.8,
    specificity=TRUE, c_specificity="uni",  p_specificity=0.5, col="red", lwd=2,
    main="cost & specificity constrain") 

n=nrow((dataBerek))
AP=sum(dataBerek$bug)
AN=n-AP
rra(berekWMCROCcurve$x, berekWMCROCcurve$y, AP, AN, j = TRUE, c_j = 0.63) 

rra.test(berekWMCROCcurve$x, berekWMCROCcurve$y, berekACROCcurve$x, berekACROCcurve$y, 
         AP, AN, precision=TRUE, fallout=TRUE, c_fallout="uni", p_fallout=0.5) 

resWMC=rra(berekWMCROCcurve$x, berekWMCROCcurve$y, plot=FALSE,
         AP, AN, precision=TRUE, fallout=TRUE, c_fallout="uni", p_fallout=0.5) 
resAC=rra(berekACROCcurve$x, berekACROCcurve$y, plot=FALSE,
         AP, AN, precision=TRUE, fallout=TRUE, c_fallout="uni", p_fallout=0.5) 
cat("RRA for WMC is ", resWMC$rra, ", RRA for AC is ", resAC$rra, "\n", sep="")

rra.test(resWMC, resAC)

resLOC=rra(berekLOCROCcurve$x, berekLOCROCcurve$y, plot=FALSE,
          AP, AN, precision=TRUE, fallout=TRUE, c_fallout="uni", p_fallout=0.5) 

rra.test(list(resWMC, resAC, resLOC),legend = TRUE, cUnderBest="white",
         main = "Test multiple RRAs", colOther="midnight blue")
